package fr.uha.gasser.pathfinding.ui;

import fr.uha.gasser.pathfinding.algorithm.AStarWorker;
import fr.uha.gasser.pathfinding.algorithm.Node;
import fr.uha.gasser.pathfinding.model.Cell;
import fr.uha.gasser.pathfinding.model.Grid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class App extends JFrame {

    private static final int ROWS = 30;
    private static final int COLS = 30;
    private static final int CELL_SIZE = 16;
    private static final Logger LOGGER = LoggerFactory.getLogger(App.class);

    private final Grid grid;
    private AStarWorker pathfinder;
    private Node startNode;
    private Node endNode;
    private JPanel gridPane;
    private boolean wallPlacing = false;

    private App() throws HeadlessException {
        super("Pathfinding");
        grid = new Grid(ROWS, COLS, CELL_SIZE);
        setupInterface();
        pack();
        setVisible(true);
    }

    private void setupInterface() {
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        gridPane = new JPanel();
        gridPane.setLayout(new GridLayout(ROWS, COLS));
        for (Cell c : grid) {
            gridPane.add(c.getContent());
        }

        gridPane.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                final JLabel at = (JLabel) gridPane.getComponentAt(e.getPoint());
                final Point loc = grid.get(at).getPos();
                if (wallPlacing) {
                    doPlaceWall(loc);
                } else if (startNode == null) {
                    grid.setColor(loc, Color.RED);
                    startNode = new Node(loc);
                } else if (endNode == null) {
                    grid.setColor(loc, Color.GREEN);
                    endNode = new Node(loc);
                }
            }
        });

        final JMenuBar menuBar = new JMenuBar();
        final JMenuItem placeWallItem = new JMenuItem("Place wall");
        menuBar.add(placeWallItem);
        placeWallItem.addActionListener(actionEvent -> setWallPlacing(placeWallItem));
        final JMenuItem runItem = new JMenuItem("Run");
        menuBar.add(runItem);
        runItem.addActionListener(actionEvent -> doSearch());
        final JMenuItem cancelItem = new JMenuItem("Cancel");
        menuBar.add(cancelItem);
        cancelItem.addActionListener(actionEvent -> doCancel());
        final JMenuItem resetItem = new JMenuItem("Reset");
        menuBar.add(resetItem);
        resetItem.addActionListener(actionEvent -> doReset());

        setJMenuBar(menuBar);
        getContentPane().add(gridPane);
    }

    private void setWallPlacing(JMenuItem placeWallItem) {
        // Invert selection state
        placeWallItem.setSelected(!placeWallItem.isSelected());
        final boolean selected = placeWallItem.isSelected();
        if (selected) {
            placeWallItem.setBackground(Color.BLUE);
        } else {
            placeWallItem.setBackground(null);
        }
        this.wallPlacing = selected;
    }

    private void doPlaceWall(Point loc) {
        final boolean wall = grid.get(loc).isWall();
        // Toggle "wall" state
        grid.setWall(loc, !wall);
    }
    
    private void doReset() {
        pathfinder = null;
        startNode = null;
        endNode = null;
        grid.clear();
    }

    private void doCancel() {
        if (pathfinder == null) return;
        pathfinder.cancel(true);
    }

    private void doSearch() {
        pathfinder = new AStarWorker(startNode, endNode, grid);
        pathfinder.execute();
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(App::new);
    }
}
